//
// RANDGEN.CPP
//
// Two, count them, two random number generators from
// various books. Completely eliminates dependency on system-supplied
// random-number generators.
// 
// The r250 generator is very fast and very random, and depends on
// the linear congruent generator for its initialization


//#include <stdlib.h>
#include "randgen.h"

RandomGenerator::RandomGenerator(unsigned int unSeed)
{
	vSeed(unSeed);
}

long RandomGenerator::lGetChecksum(void)
{
	// checksum all member variables that are important to net synch

	long lCheck = 0;
	int i;

	// do self
	for (i = 0; i < 250; ++i) {
		lCheck += maunTable[i];
	}
	
	return lCheck;
}

void RandomGenerator::vDumpSyncText(FILE *fileDump)
{
	// do self
	fprintf(fileDump, "**Global Random Checksum: %ld\n", lGetChecksum());
}

void RandomGenerator::vSeed(unsigned int unSeed)
{
	int j,
	 k;
	unsigned int mask;
	unsigned int msb;

	vLCSeed(unSeed);
	mnIndex = 0;
	for (j = 0; j < 250; j++) {	/* Fill the r250 buffer with 32-bit values */
		// the LC generator doesn't make full 32-bit values, so we'll get two
		// and XOR them together, shifted slightly
		maunTable[j] = unLCRand() ^ (unLCRand() << 8);
	}

	// Now we do a little manipulation to the values
	// in order to ensure that no column of bits is without at least one
	// 1 value, and is linearly independent.
	msb = 0x80000000;				/* To turn on the diagonal bit   */
	mask = 0xffffffff;				/* To turn off the leftmost bits */
	for (j = 0; j < 32; j++) {
		k = 5 * j + 3;			/* Select a word to operate on        */
		maunTable[k] &= mask;	/* Turn off bits left of the diagonal */
		maunTable[k] |= msb;	/* Turn on the diagonal bit           */
		mask >>= 1;
		msb >>= 1;
	}
}

int RandomGenerator::nGetRandom(int nValues) // 0 to (nValues - 1)
{
	if (nValues) {
		return (int)(unGetRandom() % nValues);
	} else {
		//vDebugOut("nGetRandom called with 0\n");
		return 0;
	}
}

int RandomGenerator::nGetRandom(int nLower, int nUpper) // nLower to nUpper, inclusive
{
	// switch lower and upper, if necessary
	if (nLower > nUpper) {
		int nTemp = nLower;
		nLower = nUpper;
		nUpper = nTemp;
	}

	int nRange = nUpper - nLower;

	// check for degenerate case where nLower == nUpper
	if (nRange) {
		return nLower + nGetRandom(nRange);
	} else {
		return nLower;
	}
}

double RandomGenerator::dGetRandom(void) // between 0.0 and 1.0
{
	return unGetRandom() / (double) 0xffffffff;
}

/***************************************************************************
*  Module:  r250.cpp   Description: implements R250 random number generator,
*  from S. Kirkpatrick and E. Stoll, Journal of Computational Physics, 40,
*  p. 517 (1981). Written by:    W. L. Maier
***************************************************************************/


/**** Static variables ****/
static unsigned int r250_buffer[250];
static int r250_index;

/**** Function: r250_init  Description: initializes r250 random number
generator. ****/
void r250_init(unsigned int unSeed)
{
	int j,
	 k;
	unsigned int mask;
	unsigned int msb;

	vLCSeed(unSeed);
	r250_index = 0;
	for (j = 0; j < 250; j++) {	/* Fill the r250 buffer with 32-bit values */
		// the LC generator doesn't make full 32-bit values, so we'll get two
		// and XOR them together, shifted slightly
		r250_buffer[j] = unLCRand() ^ (unLCRand() << 8);
	}

	msb = 0x80000000;				/* To turn on the diagonal bit   */
	mask = 0xffffffff;				/* To turn off the leftmost bits */
	for (j = 0; j < 32; j++) {
		k = 5 * j + 3;			/* Select a word to operate on        */
		r250_buffer[k] &= mask;	/* Turn off bits left of the diagonal */
		r250_buffer[k] |= msb;	/* Turn on the diagonal bit           */
		mask >>= 1;
		msb >>= 1;
	}
}

/**** Function:  r250  Description: returns a random unsigned integer. ****/
unsigned int r250()
{
/*------------------------------------------------------------------------*/
	register int j;
	register unsigned int new_rand;
/*------------------------------------------------------------------------*/
	if (r250_index >= 147)
		j = r250_index - 147;	/* Wrap pointer around */
	else
		j = r250_index + 103;

	new_rand = r250_buffer[r250_index] ^ r250_buffer[j];
	r250_buffer[r250_index] = new_rand;
	if (r250_index >= 249)		/* Increment pointer for next time */
		r250_index = 0;
	else
		r250_index++;

	return new_rand;
}

/**** Function:  r250  Description: returns a random double in range 0-1. ****/
double dr250()
{
	return (double)r250() / (double)0xffff;	/* Return a number in 0.0 to 1.0 */
}

//
// LINEAR CONGRUENT RANDOM NUMBER GENERATOR
//
// From Data Structures and Algorithm Analysis in C
//
// returns numbers in the range 0 - m,
// which is to say 31 bits of randomness

unsigned int sunSeed;				/* global variable */

#define a      16807			/* 7^5 */
#define m      2147483647		/* 2^31 - 1 */
#define q      127773			/* m/a */
#define r      2836				/* m%a */

void vLCSeed(unsigned int unSeed)
{
	sunSeed = unSeed;
}

unsigned int unLCRand(void)
{
	int tmp_seed;

	tmp_seed = a * (sunSeed % q) - r * (sunSeed / q);
	if (tmp_seed >= 0)
		sunSeed = tmp_seed;
	else
		sunSeed = tmp_seed + m;

	return sunSeed;
}

double dLCRand(void)
{
	return (((double) unLCRand()) / m);
}



